@@ -55,6 +55,7 @@ void InitConfig() {
     config.FrameNameIndex = NULL;
     config.FrameNameMail = NULL;
     config.Statistics = WMF_NOVALUE;
+    config.Extension = NULL;
 }
 
 
@@ -144,6 +145,12 @@ void DestroyConfig(void) {
 	freechar(config.FrameNameMail);
 	config.FrameNameMail = NULL;
     }
+    if (config.Extension != NULL) {
+	if (config.DebugLevel & 2)
+	    printf("..Extension\n");
+	freechar(config.Extension);
+	config.Extension = NULL;
+    }
 }
 
 
@@ -269,6 +276,11 @@ void ReadDefaults () {
     if (config.CountMessages == WMF_NOVALUE) config.CountMessages = TRUE;
     if (config.FrameSupport == WMF_NOVALUE) config.FrameSupport = FALSE;
     if (config.Statistics == WMF_NOVALUE) config.Statistics = FALSE;
+#ifdef WMF_EXTENSION
+    if (config.Extension == NULL) config.Extension = dupchar(WMF_EXTENSION);
+#endif
+    if ((config.DebugLevel != WMF_NOVALUE) && (config.DebugLevel & 2))
+        printf("..EXTENSION = %s\n",config.Extension);
 }
 
 
@@ -344,6 +356,14 @@ void ReadEnviroments () {
         	printf("..WMF_MCMD = %s\n",config.MailCommand);
         }
     }
+    if ((char *)getenv("WMF_EXTENSION") != NULL) {
+	if (config.Extension == NULL) {
+	    config.Extension = newchar(strlen((char *)getenv("WMF_EXTENSION"))+1);
+	    strncpy(config.Extension,(char *)getenv("WMF_EXTENSION"),strlen((char *)getenv("WMF_EXTENSION"))+1);
+            if ((config.DebugLevel != WMF_NOVALUE) && (config.DebugLevel & 2))
+        	printf("..WMF_EXTENSION = %s\n",config.Extension);
+        }
+    }
 }
 
 
@@ -693,6 +713,11 @@ void ReadConfigFile (char *FileName) {
 			    config.Statistics = TRUE;
 		    }
 		}
+		if (strcmp(ls_varname,"EXTENSION") == 0) {
+		    if (config.Extension == NULL) {
+			config.Extension = dupchar(ls_varvalue);
+		    }
+		}
 		/* Cleanup */
 		freechar(ls_varname);
 		freechar(ls_varvalue);
