@@ -143,9 +143,11 @@ struct MAILHEAD *ReadHtmlHeader (void) {
             (strncmp(DirEntry->d_name,"subject",7) != 0) &&
             (strncmp(DirEntry->d_name,"thread",6) != 0) &&
             (strncmp(DirEntry->d_name,"author",6) != 0) &&
+            (strncmp(DirEntry->d_name,"stats",5) != 0) &&
+            (strncmp(DirEntry->d_name,"index",5) != 0) &&
             (strcmp(DirEntry->d_name,".") != 0) &&
             (strcmp(DirEntry->d_name,"..") != 0) &&
-            (strstr(DirEntry->d_name,".html") != NULL)) {
+            (strstr(DirEntry->d_name,config.Extension) != NULL)) {
             FileName = newchar(strlen(config.Directory)
                 + strlen(DirEntry->d_name) + 2);
             sprintf(FileName,"%s/%s",config.Directory,DirEntry->d_name);
@@ -294,7 +296,7 @@ struct MAILHEAD *ReadHtmlHeader (void) {
                 }	
 		if (return_mh == NULL) return_mh = mh;
 	        if ((mh != NULL) && (config.ListLimit > 0)) {
-		    tmptime = (int) getutc(mh->SentDate);
+		    tmptime = getutc(mh->SentDate);
 		    if (tmptime < config.ListLimit) {
 			last_mh = mh->Prev;
 			if (last_mh != NULL) last_mh->Next = NULL;
@@ -312,7 +314,7 @@ struct MAILHEAD *ReadHtmlHeader (void) {
         DirEntry = readdir(dd);
     }
     if ((mh != NULL) && (config.ListLimit > 0)) {
-	tmptime = (int) getutc(mh->SentDate);
+	tmptime = getutc(mh->SentDate);
 	if (tmptime < config.ListLimit) {
 	    if (return_mh == mh) return_mh = NULL;
 	    last_mh = mh->Prev;
@@ -467,6 +469,7 @@ void strint_sort(struct MAILHEAD *mh) {
 	    }
 	}
     }
+    if (config.DebugLevel & 2) printf("..strint_sort_Return\n");
 }
 
 char *GetMainSubject(char *Subject) {
@@ -474,6 +477,7 @@ char *GetMainSubject(char *Subject) {
     int loop, i, length;
     char *tmpstr, *retstr;
 
+    if (config.DebugLevel & 1) printf(".GetMainSubject(%s)\n",Subject);
     tmpstr = Subject;
     loop = TRUE;
     while(loop) {
@@ -496,21 +500,11 @@ char *GetMainSubject(char *Subject) {
     } else {
 	retstr = dupchar(WMF_NOSUBJECT);
     }
+    if (config.DebugLevel & 2) printf("..GetMainSubject_return(%s)\n",retstr);
     return retstr;
 }
 
 
-void WriteIndexNavigation(char *FileName) {
-
-    AppendFile(FileName,"<B>Index:</B>\n");
-    if (config.IndexThread == TRUE) AppendFile(FileName,"[<A HREF=\"thread.html\">thread</A>]\n");
-    if (config.IndexDate == TRUE) AppendFile(FileName,"[<A HREF=\"date.html\">date</A>]\n");
-    if (config.IndexSubject == TRUE) AppendFile(FileName,"[<A HREF=\"subject.html\">subject</A>]\n");
-    if (config.IndexAuthor == TRUE) AppendFile(FileName,"[<A HREF=\"author.html\">author</A>]\n");
-    if (config.Statistics == TRUE) AppendFile(FileName,"[<A HREF=\"stats.html\">statistic</A>]\n");
-}
-
-
 void WriteListHead (char *Title, char *FileName, int PageNr, int MsgCount) {
     time_t BuildTime;
     char *tmpstr;
@@ -518,22 +512,26 @@ void WriteListHead (char *Title, char *FileName, int PageNr, int MsgCount) {
     if (config.DebugLevel & 1) printf(".WriteListHead\n");
 
     InitHtml(FileName);
-    AppendFile(FileName,"<HEAD>\n");
-    AppendFile(FileName,Title);
-    AppendFile(FileName,"</HEAD>\n");
-    if (HtmlCmd.BodyStart == NULL)
-	AppendFile(FileName,HTMLCMD_BODY_START);
-    else
-	AppendFile(FileName,HtmlCmd.BodyStart);
-    AppendFile(FileName,"\n");
-
-    if (config.FrameSupport == FALSE) {
-	WriteIndexNavigation(FileName);
-	if (HtmlCmd.HrStart == NULL)
-	    AppendFile(FileName,HTMLCMD_HR_START);
+    if (config.DocStartFile != NULL) {
+	IncludeFile(FileName,config.DocStartFile);
+    } else {
+	AppendFile(FileName,"<HEAD>\n");
+	AppendFile(FileName,Title);
+	AppendFile(FileName,"</HEAD>\n");
+	if (HtmlCmd.BodyStart == NULL)
+	    AppendFile(FileName,HTMLCMD_BODY_START);
 	else
-	    AppendFile(FileName,HtmlCmd.HrStart);
+	    AppendFile(FileName,HtmlCmd.BodyStart);
 	AppendFile(FileName,"\n");
+
+	if (config.FrameSupport == FALSE) {
+	    WriteIndexNavigation(FileName);
+	    if (HtmlCmd.HrStart == NULL)
+		AppendFile(FileName,HTMLCMD_HR_START);
+	    else
+		AppendFile(FileName,HtmlCmd.HrStart);
+	    AppendFile(FileName,"\n");
+	}
     }
 
     if (PageNr > 0) {
@@ -596,7 +594,6 @@ void WriteListHead (char *Title, char *FileName, int PageNr, int MsgCount) {
 
 	AppendFile(FileName,"<H2 ALIGN=LEFT>Index</H2>\n");
     }
-    AppendFile(FileName,"<UL>\n");
 }
 
 
@@ -605,7 +602,6 @@ void WriteListEnd (char *FileName, int PageNr, struct MAILHEAD *mh,
     char *tmpstr;
     
     if (config.DebugLevel & 1) printf(".WriteListEnd\n");
-    AppendFile(FileName,"</UL>\n");
     if (HtmlCmd.HrStart == NULL)
 	AppendFile(FileName,HTMLCMD_HR_START);
     else
@@ -619,25 +615,25 @@ void WriteListEnd (char *FileName, int PageNr, struct MAILHEAD *mh,
     if (PageNr > 1) {
         if (PageNr == 2) {
 	    if (config.FrameSupport == TRUE) {
-		tmpstr = newchar(strlen(ListType)+
-		  strlen(config.FrameNameIndex)+42);
-		sprintf(tmpstr,"[<A HREF=\"%s.html\" TARGET=\"%s\">previous</A>]\n",
-	          ListType,config.FrameNameIndex);
+		tmpstr = newchar(strlen(ListType)+strlen(config.Extension)+
+		  strlen(config.FrameNameIndex)+37);
+		sprintf(tmpstr,"[<A HREF=\"%s%s\" TARGET=\"%s\">previous</A>]\n",
+	          ListType,config.Extension,config.FrameNameIndex);
 	    } else {
-		tmpstr = newchar(strlen(ListType)+32);
-		sprintf(tmpstr,"[<A HREF=\"%s.html\">previous</A>]\n",
-	          ListType);
+		tmpstr = newchar(strlen(ListType)+strlen(config.Extension)+27);
+		sprintf(tmpstr,"[<A HREF=\"%s%s\">previous</A>]\n",
+	          ListType,config.Extension);
 	    }
         } else {
 	    if (config.FrameSupport == TRUE) {
 		tmpstr = newchar(strlen(ListType)+
-		  strlen(config.FrameNameIndex)+39);
-		sprintf(tmpstr,"[<A HREF=\"%s_%d.html\" TARGET=\"%s\">previous</A>]\n",
-	          ListType,(PageNr-1),config.FrameNameIndex);
+		  strlen(config.FrameNameIndex)+strlen(config.Extension)+38);
+		sprintf(tmpstr,"[<A HREF=\"%s_%d%s\" TARGET=\"%s\">previous</A>]\n",
+	          ListType,(PageNr-1),config.Extension,config.FrameNameIndex);
 	    } else {
-		tmpstr = newchar(strlen(ListType)+39);
-		sprintf(tmpstr,"[<A HREF=\"%s_%d.html\">previous</A>]\n",
-	          ListType,(PageNr-1));
+		tmpstr = newchar(strlen(ListType)+strlen(config.Extension)+28);
+		sprintf(tmpstr,"[<A HREF=\"%s_%d%s\">previous</A>]\n",
+	          ListType,(PageNr-1),config.Extension);
 	    }
         }
         AppendFile(FileName,tmpstr);
@@ -646,29 +642,39 @@ void WriteListEnd (char *FileName, int PageNr, struct MAILHEAD *mh,
     }
     if ((PageNr > 0) && (mh != NULL)) {
 	if (config.FrameSupport == TRUE) {
-	    tmpstr = newchar(strlen(ListType)+strlen(config.FrameNameIndex)+45);
-	    sprintf(tmpstr,"[<A HREF=\"%s_%d.html\" TARGET=\"%s\">next</A>]\n",
-	      ListType,(PageNr+1),config.FrameNameIndex);
+	    tmpstr = newchar(strlen(ListType)+strlen(config.FrameNameIndex)+
+	      strlen(config.Extension)+45);
+	    sprintf(tmpstr,"[<A HREF=\"%s_%d%s\" TARGET=\"%s\">next</A>]\n",
+	      ListType,(PageNr+1),config.Extension,config.FrameNameIndex);
 	} else {
-	    tmpstr = newchar(strlen(ListType)+35);
-	    sprintf(tmpstr,"[<A HREF=\"%s_%d.html\">next</A>]\n",
-	      ListType,(PageNr+1));
+	    tmpstr = newchar(strlen(ListType)+strlen(config.Extension)+35);
+	    sprintf(tmpstr,"[<A HREF=\"%s_%d%s\">next</A>]\n",
+	      ListType,(PageNr+1),config.Extension);
 	}
         AppendFile(FileName,tmpstr);
         freechar(tmpstr);
         tmpstr = NULL;
     }
     AppendFile(FileName,"<P>\n");
-    tmpstr = newchar(strlen(HREF)+strlen(NAME)+strlen(VERSION)+26);
-    sprintf(tmpstr,"<A HREF=\"%s\">%s</A> version %s\n",HREF,NAME,VERSION);
+    if (config.FrameSupport == TRUE) {
+	tmpstr = newchar(strlen(HREF)+strlen(NAME)+strlen(VERSION)+38);
+	sprintf(tmpstr,"<A HREF=\"%s\" TARGET=_top>%s</A> version %s\n",HREF,NAME,VERSION);
+    } else {
+	tmpstr = newchar(strlen(HREF)+strlen(NAME)+strlen(VERSION)+26);
+	sprintf(tmpstr,"<A HREF=\"%s\">%s</A> version %s\n",HREF,NAME,VERSION);
+    }
     AppendFile(FileName,tmpstr);
     freechar(tmpstr);
     tmpstr = NULL;
-    if (HtmlCmd.BodyEnd == NULL)
-	AppendFile(FileName,HTMLCMD_BODY_END);
-    else
-	AppendFile(FileName,HtmlCmd.BodyEnd);
-    AppendFile(FileName,"\n");
+    if (config.DocEndFile != NULL) {
+	IncludeFile(FileName,config.DocEndFile);
+    } else {
+	if (HtmlCmd.BodyEnd == NULL)
+	    AppendFile(FileName,HTMLCMD_BODY_END);
+	else
+	    AppendFile(FileName,HtmlCmd.BodyEnd);
+	AppendFile(FileName,"\n");
+    }
     EndHtml(FileName);
 }
 
@@ -689,7 +695,7 @@ struct MAILHEAD *FindId(struct MAILHEAD *mh, char *Id) {
 struct MAILHEAD *FindMainSubject(struct MAILHEAD *mh, char *Subject, char *RefID) {
 
     struct MAILHEAD *tmp_mh, *tmpLastMh, *tmpNextMh;
-    char *tmpRefId, *tmpSubject;
+    char *tmpRefId, *tmpSubject, *tmpstr;
 
     if (config.DebugLevel & 1) printf(".FindMainSubject(%s,%s)\n",Subject,RefID);
     if (config.ThreadNewSubject != TRUE) {
@@ -712,12 +718,19 @@ struct MAILHEAD *FindMainSubject(struct MAILHEAD *mh, char *Subject, char *RefID
 
 	/* Already the top one ? */
 	if (tmpLastMh != NULL) {
-	    if (config.DebugLevel & 2) printf("..TopOneRef ('%s','%s','%s')\n",
-		tmpLastMh->Subject,GetMainSubject(tmpLastMh->Subject),tmpLastMh->Reference);
-	    if ((strcmp(tmpLastMh->Subject,GetMainSubject(tmpLastMh->Subject)) == 0) &&
-		(strlen(tmpLastMh->Reference) == 0)) {
+	    tmpstr = GetMainSubject(tmpLastMh->Subject);
+	    if (config.DebugLevel & 2) {
+		printf("..TopOneRef ('%s','%s','%s')\n",
+		  tmpLastMh->Subject,tmpstr,tmpLastMh->Reference);
+	    }
+	    if ((strcmp(tmpLastMh->Subject,tmpstr) == 0) &&
+	      (strlen(tmpLastMh->Reference) == 0)) {
+		freechar(tmpstr);
+		tmpstr = NULL;
 		return tmpLastMh;
 	    }
+	    freechar(tmpstr);
+	    tmpstr = NULL;
 	}
 
 	tmp_mh = mh;
@@ -737,10 +750,13 @@ struct MAILHEAD *FindMainSubject(struct MAILHEAD *mh, char *Subject, char *RefID
 	}
 
 	if (tmp_mh != NULL) {
+	    tmpstr = GetMainSubject(tmp_mh->Subject);
 	    if (config.DebugLevel & 2) printf("..TopOneSub ('%s','%s','%s')\n",
-		tmp_mh->Subject,GetMainSubject(tmp_mh->Subject),tmp_mh->Reference);
+		tmp_mh->Subject,tmpstr,tmp_mh->Reference);
 	    if (strlen(tmp_mh->Reference) != 0)
-		tmp_mh = FindMainSubject(mh,GetMainSubject(tmp_mh->Subject),tmp_mh->Reference);
+		tmp_mh = FindMainSubject(mh,tmpstr,tmp_mh->Reference);
+	    freechar(tmpstr);
+	    tmpstr = NULL;
 	    return tmp_mh;
 	} else {
 	    return tmpLastMh;
@@ -778,11 +794,11 @@ int WriteThreadLineEx(struct MAILHEAD *mh, int page, int page_counter, int level
     }
 
     if (page > 1) {
-	FileName = newchar(strlen(config.Directory)+strlen("thread.html")+9);
-	sprintf(FileName,"%s/thread_%d.html",config.Directory,page);
+	FileName = newchar(strlen(config.Directory)+strlen(config.Extension)+19);
+	sprintf(FileName,"%s/thread_%d%s",config.Directory,page,config.Extension);
     } else {
-	FileName = newchar(strlen(config.Directory)+strlen("thread.html")+2);
-	sprintf(FileName,"%s/thread.html",config.Directory);
+	FileName = newchar(strlen(config.Directory)+strlen(config.Extension)+8);
+	sprintf(FileName,"%s/thread%s",config.Directory,config.Extension);
     }
 
     tmpName = Email2Html(mh);
@@ -875,11 +891,11 @@ int WriteThreadLine(struct MAILHEAD *mh, int page, int page_counter, int level)
     }
 
     if (page > 1) {
-	FileName = newchar(strlen(config.Directory)+strlen("thread.html")+9);
-	sprintf(FileName,"%s/thread_%d.html",config.Directory,page);
+	FileName = newchar(strlen(config.Directory)+strlen(config.Extension)+19);
+	sprintf(FileName,"%s/thread_%d%s",config.Directory,page,config.Extension);
     } else {
-	FileName = newchar(strlen(config.Directory)+strlen("thread.html")+2);
-	sprintf(FileName,"%s/thread.html",config.Directory);
+	FileName = newchar(strlen(config.Directory)+strlen(config.Extension)+8);
+	sprintf(FileName,"%s/thread%s",config.Directory,config.Extension);
     }
 
     tmpName = Email2Html(mh);
@@ -954,9 +970,10 @@ int WriteThreadLine(struct MAILHEAD *mh, int page, int page_counter, int level)
     return tmpCount;
 }
 
+
 void WriteThread (struct MAILHEAD *mh, int MsgCount) {
     struct MAILHEAD *tmp_mh, *scan_mh, *parent_mh;
-    char *FileName, *tmpstr, *tmpName, *tmpTitle, *ls_subject;
+    char *FileName, *tmpstr, *tmpName, *tmpTitle, *ls_subject, *ls_subject2;
     int page, page_counter, li_firstflag, i;
                
     if (config.DebugLevel & 1) printf(".WriteThread\n");
@@ -967,7 +984,7 @@ void WriteThread (struct MAILHEAD *mh, int MsgCount) {
 
     /* Fill sort-field */
     while (tmp_mh != NULL) {
-	tmp_mh->isort = (int) getutc(tmp_mh->SentDate);
+	tmp_mh->isort = getutc(tmp_mh->SentDate);
 	if (tmp_mh->ssort != NULL) freechar(tmp_mh->ssort);
 	tmp_mh->ssort = GetMainSubject(tmp_mh->Subject);
 	if (config.ThreadCaseSort == FALSE) {
@@ -980,8 +997,8 @@ void WriteThread (struct MAILHEAD *mh, int MsgCount) {
 
     int_sort(TopOfList(mh));
 
-    FileName = newchar(strlen(config.Directory)+strlen("thread.html")+2);
-    sprintf(FileName,"%s/thread.html",config.Directory);
+    FileName = newchar(strlen(config.Directory)+strlen(config.Extension)+8);
+    sprintf(FileName,"%s/thread%s",config.Directory,config.Extension);
     if (config.Name != NULL) {
         tmpTitle = newchar(strlen(config.Name)+26);
         sprintf(tmpTitle,"<TITLE>%s threaded</TITLE>\n",config.Name);
@@ -997,20 +1014,26 @@ void WriteThread (struct MAILHEAD *mh, int MsgCount) {
     }
 
     WriteListHead(tmpTitle,FileName,page,MsgCount);
+    AppendFile(FileName,"<UL>\n");
 
     tmp_mh = TopOfList(mh);
     while (tmp_mh != NULL) {
         if (tmp_mh->Printed == FALSE) {
-            parent_mh = FindMainSubject(TopOfList(mh),GetMainSubject(tmp_mh->Subject),tmp_mh->Reference);
+            tmpstr = GetMainSubject(tmp_mh->Subject);
+            parent_mh = FindMainSubject(TopOfList(mh),tmpstr,tmp_mh->Reference);
+            freechar(tmpstr);
+            tmpstr = NULL;
 	    if (config.PageSize > 0) {
 	        if ((page_counter >= config.PageSize) && (tmp_mh != NULL)) {
+		    AppendFile(FileName,"</UL>\n");
 	 	    WriteListEnd(FileName,page,tmp_mh,"thread");
 		    page_counter = 0;
 		    page++;
 		    freechar(FileName);
-		    FileName = newchar(strlen(config.Directory)+strlen("thread.html")+9);
-		    sprintf(FileName,"%s/thread_%d.html",config.Directory,page);
+		    FileName = newchar(strlen(config.Directory)+strlen(config.Extension)+19);
+		    sprintf(FileName,"%s/thread_%d%s",config.Directory,page,config.Extension);
 		    WriteListHead(tmpTitle,FileName,page,MsgCount);
+		    AppendFile(FileName,"<UL>\n");
 	        }
 	        page_counter++;
 	    }
@@ -1019,6 +1042,7 @@ void WriteThread (struct MAILHEAD *mh, int MsgCount) {
 		tmpstr = newchar(strlen(ls_subject)+7);
 		sprintf(tmpstr,"<LI> %s\n",ls_subject);
 		freechar(ls_subject);
+		ls_subject = NULL;
 		AppendFile(FileName,tmpstr);
 		freechar(tmpstr);
 		tmpstr = NULL;
@@ -1045,8 +1069,32 @@ void WriteThread (struct MAILHEAD *mh, int MsgCount) {
 				page_counter = WriteThreadLine(scan_mh,page,page_counter,1);
 			scan_mh = scan_mh->Next;
 		    }
- 		    AppendFile(FileName,"</UL>\n");
+ 		} else {
+		    tmpName = Email2Html(tmp_mh);
+		    ls_subject2 = Text_2_Html(tmp_mh->Subject);
+	            if (config.FrameSupport == TRUE) {
+			tmpstr = newchar(strlen(tmp_mh->FileName)+
+			  strlen(ls_subject2)+strlen(tmpName)+
+			  strlen(config.FrameNameMail)+35);
+			sprintf(tmpstr,"<LI> <A HREF=\"%s\" TARGET=\"%s\">%s</A> (%s)\n",
+			  tmp_mh->FileName,config.FrameNameMail,
+			  ls_subject2,tmpName);
+		    } else {
+			tmpstr = newchar(strlen(tmp_mh->FileName)+
+			  strlen(ls_subject2)+strlen(tmpName)+25);
+			sprintf(tmpstr,"<LI> <A HREF=\"%s\">%s</A> (%s)\n",
+			  tmp_mh->FileName,ls_subject2,tmpName);
+		    }
+		    AppendFile(FileName,tmpstr);
+		    freechar(ls_subject2);
+		    ls_subject2 = NULL;
+		    freechar(tmpstr);
+		    tmpstr = NULL;
+		    freechar(tmpName);
+		    tmpName = NULL;
+		    tmp_mh->Printed = TRUE;
  		}
+ 		AppendFile(FileName,"</UL>\n");
             } else {
 		li_firstflag = TRUE;
 		tmpName = Email2Html(parent_mh);
@@ -1135,6 +1183,7 @@ void WriteThread (struct MAILHEAD *mh, int MsgCount) {
         } /* end of if */
 	tmp_mh = tmp_mh->Next;
     } /* end of while */
+    AppendFile(FileName,"</UL>\n");
     WriteListEnd(FileName,page,tmp_mh,"thread");
 
     /* Do a little cleanup */
@@ -1161,7 +1210,7 @@ void WriteSubject (struct MAILHEAD *mh, int MsgCount) {
     /* Fill sort-field */
     while (tmp_mh != NULL) {
 	if (tmp_mh->ssort != NULL) freechar(tmp_mh->ssort);
-	tmp_mh->isort = (int) getutc(tmp_mh->SentDate);
+	tmp_mh->isort = getutc(tmp_mh->SentDate);
 	tmp_mh->ssort = GetMainSubject(tmp_mh->Subject);
 	if (config.SubjectCaseSort == FALSE) {
 	    /* Convert to uppercase */
@@ -1173,8 +1222,8 @@ void WriteSubject (struct MAILHEAD *mh, int MsgCount) {
 
     strint_sort(TopOfList(mh));
 
-    FileName = newchar(strlen(config.Directory)+strlen("subject.html")+2);
-    sprintf(FileName,"%s/subject.html",config.Directory);
+    FileName = newchar(strlen(config.Directory)+strlen(config.Extension)+9);
+    sprintf(FileName,"%s/subject%s",config.Directory,config.Extension);
     if (config.Name != NULL) {
         tmpTitle = newchar(strlen(config.Name)+35);
         sprintf(tmpTitle,"<TITLE>%s sorted by subject</TITLE>\n",config.Name);
@@ -1194,7 +1243,9 @@ void WriteSubject (struct MAILHEAD *mh, int MsgCount) {
     tmp_mh = TopOfList(mh);
     while (tmp_mh != NULL) {
 	if (tmpSubject == NULL) {
-	    ls_subject = Text_2_Html(GetMainSubject(tmp_mh->Subject));
+	    tmpstr = GetMainSubject(tmp_mh->Subject);
+	    ls_subject = Text_2_Html(tmpstr);
+	    freechar(tmpstr);
 	    tmpstr = newchar(strlen(ls_subject)+11);
 	    sprintf(tmpstr,"<H3>%s</H3>\n",ls_subject);
             AppendFile(FileName,tmpstr);
@@ -1206,7 +1257,9 @@ void WriteSubject (struct MAILHEAD *mh, int MsgCount) {
             stats.Subject = 1;
 	} else {
 	    if (strcmp(tmp_mh->ssort,tmpSubject) != 0) {
-		ls_subject = Text_2_Html(GetMainSubject(tmp_mh->Subject));
+		tmpstr = GetMainSubject(tmp_mh->Subject);
+		ls_subject = Text_2_Html(tmpstr);
+		freechar(tmpstr);
 	        tmpstr = newchar(strlen(ls_subject)+11);
 	        sprintf(tmpstr,"<H3>%s</H3>\n",ls_subject);
 		AppendFile(FileName,tmpstr);
@@ -1243,8 +1296,8 @@ void WriteSubject (struct MAILHEAD *mh, int MsgCount) {
 		page_counter = 0;
         	page++;
         	freechar(FileName);
-		FileName = newchar(strlen(config.Directory)+strlen("subject.html")+9);
-		sprintf(FileName,"%s/subject_%d.html",config.Directory,page);
+		FileName = newchar(strlen(config.Directory)+strlen(config.Extension)+20);
+		sprintf(FileName,"%s/subject_%d%s",config.Directory,page,config.Extension);
 		WriteListHead(tmpTitle,FileName,page,MsgCount);
 	    }
 	    page_counter++;
@@ -1257,10 +1310,14 @@ void WriteSubject (struct MAILHEAD *mh, int MsgCount) {
 	freechar(tmpSubject);
 	tmpSubject = NULL;
     }
-    freechar(FileName);
-    FileName = NULL;
-    freechar(tmpTitle);
-    tmpTitle = NULL;
+    if (FileName != NULL) {
+	freechar(FileName);
+	FileName = NULL;
+    }
+    if (tmpTitle != NULL) {
+	freechar(tmpTitle);
+	tmpTitle = NULL;
+    }
 }
 
 
@@ -1280,7 +1337,7 @@ void WriteAuthor (struct MAILHEAD *mh, int MsgCount) {
     /* Fill sort-field */
     while (tmp_mh != NULL) {
         if (tmp_mh->ssort != NULL) freechar(tmp_mh->ssort);
-        tmp_mh->isort = (int) getutc(tmp_mh->SentDate);
+        tmp_mh->isort = getutc(tmp_mh->SentDate);
 	if ((strlen(tmp_mh->SendName) == 0) || (config.AuthorEmailSort == TRUE)) {
 	    tmp_mh->ssort = dupchar(tmp_mh->SendAddress);
 	} else {
@@ -1295,8 +1352,8 @@ void WriteAuthor (struct MAILHEAD *mh, int MsgCount) {
     }
     strint_sort(TopOfList(mh));
 
-    FileName = newchar(strlen(config.Directory)+strlen("author.html")+2);
-    sprintf(FileName,"%s/author.html",config.Directory);
+    FileName = newchar(strlen(config.Directory)+strlen(config.Extension)+8);
+    sprintf(FileName,"%s/author%s",config.Directory,config.Extension);
     if (config.Name != NULL) {
         tmpTitle = newchar(strlen(config.Name)+34);
         sprintf(tmpTitle,"<TITLE>%s sorted by author</TITLE>\n",config.Name);
@@ -1386,8 +1443,8 @@ void WriteAuthor (struct MAILHEAD *mh, int MsgCount) {
 		page_counter = 0;
         	page++;
         	freechar(FileName);
-		FileName = newchar(strlen(config.Directory)+strlen("author.html")+9);
-		sprintf(FileName,"%s/author_%d.html",config.Directory,page);
+		FileName = newchar(strlen(config.Directory)+strlen(config.Extension)+19);
+		sprintf(FileName,"%s/author_%d%s",config.Directory,page,config.Extension);
 		WriteListHead(tmpTitle,FileName,page,MsgCount);
 	    }
 	    page_counter++;
@@ -1411,50 +1468,55 @@ void WriteAuthor (struct MAILHEAD *mh, int MsgCount) {
 
 void WriteDate (struct MAILHEAD *mh, int MsgCount) {
     struct MAILHEAD *tmp_mh;
-    char *FileName, *tmpstr, *tmpName, *tmpTitle, *ls_subject;
+    char *FileName, *tmpstr, *tmpName, *tmpTitle, *ls_subject, *ls_hellbender;
     int page, page_counter;
            
     if (config.DebugLevel & 1) printf(".WriteDate\n");
 
+    ls_hellbender = newchar(1024);
     page_counter = 0;
     tmp_mh = TopOfList(mh);
 
     /* Fill sort-field */
     while (tmp_mh != NULL) {
-	tmp_mh->isort = (int) getutc(tmp_mh->SentDate);
+	tmp_mh->isort = getutc(tmp_mh->SentDate);
 	tmp_mh = tmp_mh->Next;
     }
     int_sort(TopOfList(mh));
 
     if (config.SortOrder == SORT_REVERSE) {
 	tmp_mh = BottomOfList(mh);
-	if (tmp_mh != NULL)
+	if (tmp_mh != NULL) {
 	    config.ListEnd = tmp_mh->isort;
-	else
+	} else {
 	    config.ListEnd = 0;
+	}
 	stats.First = config.ListEnd;
 	tmp_mh = TopOfList(mh);
-	if (tmp_mh != NULL)
+	if (tmp_mh != NULL) {
 	    config.ListStart = tmp_mh->isort;
-	else
+	} else {
 	    config.ListStart = 0;
+	}
 	stats.Last = config.ListStart;
     } else {
 	tmp_mh = TopOfList(mh);
-	if (tmp_mh != NULL)
+	if (tmp_mh != NULL) {
 	    config.ListStart = tmp_mh->isort;
-	else
+	} else {
 	    config.ListStart = 0;
+	}
 	stats.First = config.ListStart;
 	tmp_mh = BottomOfList(mh);
-	if (tmp_mh != NULL)
+	if (tmp_mh != NULL) {
 	    config.ListEnd = tmp_mh->isort;
-	else
+	} else {
 	    config.ListEnd = 0;
+	}
 	stats.Last = config.ListEnd;
     }
-    FileName = newchar(strlen(config.Directory)+strlen("date.html")+2);
-    sprintf(FileName,"%s/date.html",config.Directory);
+    FileName = newchar(strlen(config.Directory)+strlen(config.Extension)+6);
+    sprintf(FileName,"%s/date%s",config.Directory,config.Extension);
     if (config.Name != NULL) {
         tmpTitle = newchar(strlen(config.Name)+32);
         sprintf(tmpTitle,"<TITLE>%s sorted by date</TITLE>\n",config.Name);
@@ -1477,14 +1539,15 @@ void WriteDate (struct MAILHEAD *mh, int MsgCount) {
         ls_subject = Text_2_Html(tmp_mh->Subject);
         if (config.FrameSupport == TRUE) {
 	    tmpstr = newchar(strlen(tmp_mh->FileName)+strlen(ls_subject)+
-	      strlen(tmpName)+strlen(config.FrameNameMail)+34);
-	    sprintf(tmpstr,"<A HREF=\"%s\" TARGET=\"%s\">%s</A> (%s)<BR>\n",
-	      tmp_mh->FileName,config.FrameNameMail,ls_subject,tmpName);
+	      strlen(tmpName)+strlen(config.FrameNameMail)+
+	      strlen(tmp_mh->SentDate)+42);
+	    sprintf(tmpstr,"<A HREF=\"%s\" TARGET=\"%s\">%s</A> (%s) <I>%s</I><BR>\n",
+	      tmp_mh->FileName,config.FrameNameMail,ls_subject,tmpName,tmp_mh->SentDate);
         } else {
-	    tmpstr = newchar(strlen(tmp_mh->FileName)+
-	      strlen(ls_subject)+strlen(tmpName)+24);
-	    sprintf(tmpstr,"<A HREF=\"%s\">%s</A> (%s)<BR>\n",tmp_mh->FileName,
-	      ls_subject,tmpName);
+	    tmpstr = newchar(strlen(tmp_mh->FileName)+strlen(ls_subject)+
+	      strlen(tmpName)+strlen(tmp_mh->SentDate)+32);
+	    sprintf(tmpstr,"<A HREF=\"%s\">%s</A> (%s) <I>%s</I><BR>\n",
+	      tmp_mh->FileName,ls_subject,tmpName,tmp_mh->SentDate);
         }
         AppendFile(FileName,tmpstr);
         freechar(ls_subject);
@@ -1498,10 +1561,10 @@ void WriteDate (struct MAILHEAD *mh, int MsgCount) {
 	    if ((page_counter >= config.PageSize) && (tmp_mh != NULL)) {
 		WriteListEnd(FileName,page,tmp_mh,"date");
 		page_counter = 0;
-        	page++;
+		page++;
         	freechar(FileName);
-		FileName = newchar(strlen(config.Directory)+strlen("date.html")+9);
-		sprintf(FileName,"%s/date_%d.html",config.Directory,page);
+		FileName = newchar(strlen(config.Directory)+strlen(config.Extension)+17);
+		sprintf(FileName,"%s/date_%d%s",config.Directory,page,config.Extension);
 		WriteListHead(tmpTitle,FileName,page,MsgCount);
 	    }
 	    page_counter++;
@@ -1520,8 +1583,8 @@ void WriteStats (struct MAILHEAD *mh) {
     time_t BuildTime;
 
     if (stats.Messages == WMF_NOVALUE) stats.Messages = MessageCount(mh);
-    FileName = newchar(strlen(config.Directory)+strlen("stats.html")+2);
-    sprintf(FileName,"%s/stats.html",config.Directory);
+    FileName = newchar(strlen(config.Directory)+strlen(config.Extension)+7);
+    sprintf(FileName,"%s/stats%s",config.Directory,config.Extension);
 
     if (config.Name != NULL) {
         tmpstr = newchar(strlen(config.Name)+32);
@@ -1531,15 +1594,20 @@ void WriteStats (struct MAILHEAD *mh) {
     }
 
     InitHtml(FileName);
-    AppendFile(FileName,"<HEAD>\n");
-    AppendFile(FileName,tmpstr);
+    if (config.DocStartFile != NULL) {
+	IncludeFile(FileName,config.DocStartFile);
+    } else {
+	AppendFile(FileName,"<HEAD>\n");
+	AppendFile(FileName,tmpstr);
+	AppendFile(FileName,"</HEAD>\n");
+	if (HtmlCmd.BodyStart == NULL)
+	    AppendFile(FileName,HTMLCMD_BODY_START);
+	else
+	    AppendFile(FileName,HtmlCmd.BodyStart);
+	AppendFile(FileName,"\n");
+    }
     freechar(tmpstr);
-    AppendFile(FileName,"</HEAD>\n");
-    if (HtmlCmd.BodyStart == NULL)
-	AppendFile(FileName,HTMLCMD_BODY_START);
-    else
-	AppendFile(FileName,HtmlCmd.BodyStart);
-    AppendFile(FileName,"\n");
+    tmpstr = NULL;
     if (config.FrameSupport == FALSE) {
 	WriteIndexNavigation(FileName);
 	if (HtmlCmd.HrStart == NULL)
@@ -1558,59 +1626,59 @@ void WriteStats (struct MAILHEAD *mh) {
     AppendFile(FileName,"<TABLE>\n");
 
     if (stats.Messages != WMF_NOVALUE) {
-	AppendFile(FileName,"<TR><TH ALIGN=LEFT>Messages:<TD>");
+	AppendFile(FileName,"<TR><TH ALIGN=LEFT>Messages:</TH><TD>");
 	tmpstr = newchar(10); /* 10 for amount of messages */
 	sprintf(tmpstr,"%d",stats.Messages);
 	AppendFile(FileName,tmpstr);
 	freechar(tmpstr);
-	AppendFile(FileName,"</TR>\n");
+	AppendFile(FileName,"</TD></TR>\n");
     }
     
     if (stats.First != WMF_NOVALUE) {
-	AppendFile(FileName,"<TR><TH ALIGN=LEFT>First message:<TD>");
+	AppendFile(FileName,"<TR><TH ALIGN=LEFT>First message:</TH><TD>");
 	tmpstr = newchar(50); /* 50 for date */
 	strftime(tmpstr,50,"%a, %d. %b %Y %H:%M:%S",
 	  localtime(&stats.First));
 	AppendFile(FileName,tmpstr);
 	freechar(tmpstr);
-	AppendFile(FileName,"</TR>\n");
+	AppendFile(FileName,"</TD></TR>\n");
     }
 
     if (stats.Last != WMF_NOVALUE) {
-	AppendFile(FileName,"<TR><TH ALIGN=LEFT>Last message:<TD>");
+	AppendFile(FileName,"<TR><TH ALIGN=LEFT>Last message:</TH><TD>");
 	tmpstr = newchar(50); /* 50 for date */
 	strftime(tmpstr,50,"%a, %d. %b %Y %H:%M:%S",
 	  localtime(&stats.Last));
 	AppendFile(FileName,tmpstr);
 	freechar(tmpstr);
-	AppendFile(FileName,"</TR>\n");
+	AppendFile(FileName,"</TD></TR>\n");
     }
 
     if (stats.Author != WMF_NOVALUE) {
-	AppendFile(FileName,"<TR><TH ALIGN=LEFT>Authors:<TD>");
+	AppendFile(FileName,"<TR><TH ALIGN=LEFT>Authors:</TH><TD>");
 	tmpstr = newchar(10); /* 10 for amount of messages */
 	sprintf(tmpstr,"%d",stats.Author);
 	AppendFile(FileName,tmpstr);
 	freechar(tmpstr);
-	AppendFile(FileName,"</TR>\n");
+	AppendFile(FileName,"</TD></TR>\n");
     }
     
     if (stats.Subject != WMF_NOVALUE) {
-	AppendFile(FileName,"<TR><TH ALIGN=LEFT>Subjects:<TD>");
+	AppendFile(FileName,"<TR><TH ALIGN=LEFT>Subjects:</TH><TD>");
 	tmpstr = newchar(10); /* 10 for amount of messages */
 	sprintf(tmpstr,"%d",stats.Subject);
 	AppendFile(FileName,tmpstr);
 	freechar(tmpstr);
-	AppendFile(FileName,"</TR>\n");
+	AppendFile(FileName,"</TD></TR>\n");
     }
     
     if (stats.Thread != WMF_NOVALUE) {
-	AppendFile(FileName,"<TR><TH ALIGN=LEFT>Threads:<TD>");
+	AppendFile(FileName,"<TR><TH ALIGN=LEFT>Threads:</TH><TD>");
 	tmpstr = newchar(10); /* 10 for amount of messages */
 	sprintf(tmpstr,"%d",stats.Thread);
 	AppendFile(FileName,tmpstr);
 	freechar(tmpstr);
-	AppendFile(FileName,"</TR>\n");
+	AppendFile(FileName,"</TD></TR>\n");
     }
     
     AppendFile(FileName,"</TABLE>\n");
@@ -1624,16 +1692,25 @@ void WriteStats (struct MAILHEAD *mh) {
 	WriteIndexNavigation(FileName);
     }
     AppendFile(FileName,"<P>\n");
-    tmpstr = newchar(strlen(HREF)+strlen(NAME)+strlen(VERSION)+26);
-    sprintf(tmpstr,"<A HREF=\"%s\">%s</A> version %s\n",HREF,NAME,VERSION);
+    if (config.FrameSupport == TRUE) {
+	tmpstr = newchar(strlen(HREF)+strlen(NAME)+strlen(VERSION)+38);
+	sprintf(tmpstr,"<A HREF=\"%s\" TARGET=_top>%s</A> version %s\n",HREF,NAME,VERSION);
+    } else {
+	tmpstr = newchar(strlen(HREF)+strlen(NAME)+strlen(VERSION)+26);
+	sprintf(tmpstr,"<A HREF=\"%s\">%s</A> version %s\n",HREF,NAME,VERSION);
+    }
     AppendFile(FileName,tmpstr);
     freechar(tmpstr);
     tmpstr = NULL;
-    if (HtmlCmd.BodyEnd == NULL)
-	AppendFile(FileName,HTMLCMD_BODY_END);
-    else
-	AppendFile(FileName,HtmlCmd.BodyEnd);
-    AppendFile(FileName,"\n");
+    if (config.DocEndFile != NULL) {
+	IncludeFile(FileName,config.DocEndFile);
+    } else {
+	if (HtmlCmd.BodyEnd == NULL)
+	    AppendFile(FileName,HTMLCMD_BODY_END);
+	else
+	    AppendFile(FileName,HtmlCmd.BodyEnd);
+	AppendFile(FileName,"\n");
+    }
     EndHtml(FileName);
     freechar(FileName);
     
