@@ -32,6 +32,7 @@ char *dupchar(char *string) {
     
     if ((config.DebugLevel != WMF_NOVALUE) && (config.DebugLevel & 8))
 	printf("....dupchar(%s)\n",string);
+    if (string == NULL) return NULL;
     tmpstr = (char *) strdup(string);
     if (tmpstr == NULL) {
 	fprintf(stderr,"* not enought memory to allocate another %d bytes *\n",strlen(string));
@@ -51,6 +52,7 @@ void freechar(char *string) {
 #ifdef DEBUG
     Debug.StrDeAlloc++;
 #endif
+    if (string == NULL) return;
     free(string);
 }
 #endif
@@ -58,4 +60,4 @@ void freechar(char *string) {
 int isoctal(char number) {
     if ((number >= '0') && (number <= '9')) return TRUE;
     return FALSE;
-}
\ No newline at end of file
+}
